__version__ = "0.1.4"
__git_commiter_name__ = "Guy Rozendorn"
__git_commiter_email__ = "guy@rzn.co.il"
__git_branch__ = '(detached from v0.1.4)'
__git_remote_tracking_branch__ = '(No remote tracking)'
__git_remote_url__ = '(Not remote tracking)'
__git_head_hash__ = 'abdc897bec6ca411c606fae3f31a9d1117909a6d'
__git_head_subject__ = 'Finished release v0.1.4.'
__git_head_message__ = '* release/v0.1.4:\n  buildout.cfg: adding xmltodict==0.7.0 to requirements\n  buildout.cfg: remove xmltodict from requirements'
__git_dirty_diff__ = ''